/*
 * Copyright (c) 2004, Folding Rain Filmworks Inc.
 *
 * author: Joe Howes (joeh@guild1.com)
 *
 * Sometimes Vegas' Smart Resample doesn't do a great job of determining
 * when a video event in fact needs to be resampled.
 *
 * One area it fails in is taking certain Quicktime media in PAL format
 * and placing them in an NTSC project or rendering them out as NTSC.
 * Sometimes it's ok, but if you've ever used Quicktime media captured
 * by another NLE (FCP, for instance), you'll notice severe
 * flicker on your output monitor even if you check the "Reduce Interlace
 * Flicker" switch.  This can be a HUGE problem if you've got a large
 * project in multiple files and you'll be outputting various formats
 * (web, NTSC, PAL).
 *
 * The Flicker Buddy is a quick way to set your flicker and resample
 * settings for either selected media or media with a certain type of
 * extension (of course it defaults to 'mov' since I'm using it in a
 * setting with lots of media captured in FCP).
 *
 * In my case, going from PAL Quicktimes captured in FCP and rendering
 * to NTSC on Vegas, I had to reduce interlace flicker and disable
 * resampling.  All other media (AVI, other animations) were fine.
 * When outputting PAL, I wanted to turn off 'Reduce Interlace Flicker'
 * and enable 'Smart Resample' to get the render times back down.
 *
 *
 * USAGE:
 * If you have a certain set of media on the timeline you want to affect,
 * select them and run the script.  If you want to affect all media of a
 * certain extension, just run the script.  Set your options in the dialog
 * box and click 'Apply'.
 *
 * v1.0: Oct. 23, 2004
 */

import System.Windows.Forms;
import System.Text;
import System.Collections;
import Sony.Vegas;


/**
 * MAIN
 */
try {

	var done = false;						// If nothing is done, let the user know
	var dlog = new FlickerBuddyDialog();	// The GUI


	// Show the GUI
	if (DialogResult.OK == dlog.ShowDialog()) {

		// For each track
		for (var track in Vegas.Project.Tracks) {

			// If the track is not video, skip it
			if (!track.IsVideo()) {
				continue;
			}

			// For each event
			for (var evnt in track.Events) {

				var go = false;
				var filePath = evnt.ActiveTake.Media.FilePath;

				// If the user wants to affect selected events, and this
				// event is selected, GO.
				if (dlog.applyToSelectedEventsRadio.Checked && evnt.Selected) {

					go = true;

				// Otherwise if they want to affect certain file types and this
				// event is that type, GO.
				} else if (dlog.applyToEventsWExtRadio &&
					filePath.EndsWith(dlog.extensionTextBox.Text)) {

					go = true;

				}

				if (!go) {
					continue;
				}

				// Update the settings
				done = true;
				evnt.ReduceInterlace = dlog.reduceInterlaceFlicker.Checked;

				if (dlog.smartResampleRadio.Checked) {

					evnt.ResampleMode = VideoResampleMode.Smart;

				} else if (dlog.forceResampleRadio.Checked) {

					evnt.ResampleMode = VideoResampleMode.Force;

				} else if (dlog.disableResampleRadio.Checked) {

					evnt.ResampleMode = VideoResampleMode.Disable;

				}

			}

		}

	}


	// Only bother the user if no events or media were changed.
	if (!done) {
		MessageBox.Show("No events affected.", "Done!");
	}

} catch (e) {
    MessageBox.Show(e);
}



/**
 * CLASS: FlickerBuddyDialog
 * The GUI.
 */
class FlickerBuddyDialog extends Form {

	var applyToSelectedEventsRadio : RadioButton;
	var applyToEventsWExtRadio : RadioButton;
	var extensionTextBox : TextBox;

	var reduceInterlaceFlicker : CheckBox;
	var smartResampleRadio : RadioButton;
	var forceResampleRadio : RadioButton;
	var disableResampleRadio : RadioButton;

	var addFXButton : Button;
    var removeFXButton : Button;

    var applyButton : Button;
    var cancelButton : Button;

    var eventsGroup : GroupBox;
    var settingsGroup : GroupBox;


	/**
	 * CONSTRUCTOR
	 * Init everything.
	 */
    function FlickerBuddyDialog() {

    	this.Text = "Flicker Buddy";
		this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
		this.MaximizeBox = false;
		this.StartPosition = FormStartPosition.CenterScreen;

		this.Width = 272;
		this.Height = 265;


		// Events
		applyToSelectedEventsRadio = new RadioButton();
		applyToSelectedEventsRadio.Location = new System.Drawing.Point(10, 20);
		applyToSelectedEventsRadio.Size = new System.Drawing.Size(200, 16);
		applyToSelectedEventsRadio.Name = "applyToSelectedEventsRadio";
		applyToSelectedEventsRadio.TabIndex = 1;
		applyToSelectedEventsRadio.add_Click(this.selectedEventsOnClick);
		applyToSelectedEventsRadio.Text = "Apply to Selected Events";

		applyToEventsWExtRadio = new RadioButton();
		applyToEventsWExtRadio.Location = new System.Drawing.Point(10, 40);
		applyToEventsWExtRadio.Size = new System.Drawing.Size(180, 16);
		applyToEventsWExtRadio.Name = "applyToEventsWExtRadio";
		applyToEventsWExtRadio.TabIndex = 1;
		applyToEventsWExtRadio.Checked = true;
		applyToEventsWExtRadio.add_Click(this.eventsWExtOnClick);
		applyToEventsWExtRadio.Text = "Apply to Events with Extension: ";

		extensionTextBox = new TextBox();
		extensionTextBox.Location = new System.Drawing.Point(190, 38);
		extensionTextBox.Size = new System.Drawing.Size(45, 16);
		extensionTextBox.Name = "extensionTextBox";
		extensionTextBox.TabIndex = 1;
		extensionTextBox.Text = "mov";


		// Events ParamGroup
		eventsGroup = new GroupBox();
		eventsGroup.Controls.Add(applyToSelectedEventsRadio);
		eventsGroup.Controls.Add(applyToEventsWExtRadio);
		eventsGroup.Controls.Add(extensionTextBox);
		eventsGroup.Location = new System.Drawing.Point(8, 8);
		eventsGroup.Name = "eventsGroup";
		eventsGroup.Size = new System.Drawing.Size(250, 70);
		eventsGroup.TabIndex = 4;
		eventsGroup.TabStop = false;
		eventsGroup.Text = "Events";

		this.Controls.Add(eventsGroup);


		// Settings
		reduceInterlaceFlicker = new CheckBox();
		reduceInterlaceFlicker.Location = new System.Drawing.Point(10, 20);
		reduceInterlaceFlicker.Size = new System.Drawing.Size(145, 18);
		reduceInterlaceFlicker.Name = "replaceExistingFXCheck";
		reduceInterlaceFlicker.TabIndex = 6;
		reduceInterlaceFlicker.Checked = true;
		reduceInterlaceFlicker.Text = "Reduce Interlace Flicker";

		smartResampleRadio = new RadioButton();
		smartResampleRadio.Location = new System.Drawing.Point(10, 50);
		smartResampleRadio.Size = new System.Drawing.Size(200, 16);
		smartResampleRadio.Name = "smartResampleRadio";
		smartResampleRadio.TabIndex = 1;
		smartResampleRadio.add_Click(this.selectedEventsOnClick);
		smartResampleRadio.Text = "Smart Resample";

		forceResampleRadio = new RadioButton();
		forceResampleRadio.Location = new System.Drawing.Point(10, 70);
		forceResampleRadio.Size = new System.Drawing.Size(200, 16);
		forceResampleRadio.Name = "forceResampleRadio";
		forceResampleRadio.TabIndex = 1;
		forceResampleRadio.add_Click(this.selectedEventsOnClick);
		forceResampleRadio.Text = "Force Resample";

		disableResampleRadio = new RadioButton();
		disableResampleRadio.Location = new System.Drawing.Point(10, 90);
		disableResampleRadio.Size = new System.Drawing.Size(200, 16);
		disableResampleRadio.Name = "disableResampleRadio";
		disableResampleRadio.TabIndex = 1;
		disableResampleRadio.add_Click(this.selectedEventsOnClick);
		disableResampleRadio.Checked = true;
		disableResampleRadio.Text = "Disable Resample";


		// Settings ParamGroup
		settingsGroup = new GroupBox();
		settingsGroup.Controls.Add(reduceInterlaceFlicker);
		settingsGroup.Controls.Add(smartResampleRadio);
		settingsGroup.Controls.Add(forceResampleRadio);
		settingsGroup.Controls.Add(disableResampleRadio);
		settingsGroup.Location = new System.Drawing.Point(8, 90);
		settingsGroup.Name = "settingsGroup";
		settingsGroup.Size = new System.Drawing.Size(250, 115);
		settingsGroup.TabIndex = 4;
		settingsGroup.TabStop = false;
		settingsGroup.Text = "Events";

		this.Controls.Add(settingsGroup);


		// Apply button
        applyButton = new Button();
        applyButton.DialogResult = System.Windows.Forms.DialogResult.OK;
		applyButton.Text = "Assign";
		applyButton.Left = 100;
		applyButton.Top = 210;
		AcceptButton = applyButton;
        Controls.Add(applyButton);

		// Cancel button
        cancelButton = new Button();
		cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
		cancelButton.Text = "Cancel";
		cancelButton.Left = 182;
		cancelButton.Top = 210;
		CancelButton = cancelButton;
        Controls.Add(cancelButton);

    }


    /**
	 * FUNCTION: selectedEventsOnClick
	 */
	function selectedEventsOnClick(sender : Object, evt : System.EventArgs) {

		extensionTextBox.Enabled = false;

	}


    /**
	 * FUNCTION: eventsWExtOnClick
	 */
	function eventsWExtOnClick(sender : Object, evt : System.EventArgs) {

		extensionTextBox.Enabled = true;

	}

}

