/*
 * split "grouped" events under cursor or remove+ripple them if at left of cursor
 *
 * Main purpose: An effective tool to remove unwanted passages out of one
 * large event (like commercials), or alternatively to keep only few wanted
 * passaged out of one large event (eg for constructing trailers). Works by
 * splitting and removing events (without crossfades, only sharp cuts)
 *
 * Workflow (scenario: you want to remove commercials):
 * Drag large clip to timeline (can consist of video and audio
 * events, as long as they start and end at same timecodes on timeline, even
 * if spread over several tracks; usually it is one video and one audio event).
 * Then position cursor at start of event and scrub through it (preferrably
 * using a jogwheel) until you reach the start of a commercial. run script
 * (preferrably by pressing Ctrl-1 to Ctrl-9 or button on jogwheel). This
 * will split the clip at this location. Then scrub forward until you reach
 * the end of the comercial (irst frame of movie afterwards). Then run
 * the script TWICE (pressing the button twice). First run will again split
 * the event, but the second run will actually remove the left event,
 * because - due to the first run - the cursor is now precisely at the right
 * edge of the event. Also, the script automatically ripples all remaining
 * events to the right, no matter whether ripple mode is on or not, and
 * positions the cursor at the start of the removed event (so the cursor
 * moves to the left). Nothe though that the timeline doesn't scroll - this
 * cannot be done within scripts, so the cursor eventually moves outside the
 * timeline window (to its left). If so, a press to teh right arrow (or
 * scrubbing in any way, preferrably using the jogwheel) will "adjust" the
 * timeline preview window so the cursor becomes again visible.
 * Loop the process for next comercial.
 * For construcing trailers, same procedure can be applied.
 *
 * Basically, the script EITHER splits several events under the cursor, OR
 * it removes several events to the left of the cursor. It NEVER mixes the
 * two actions! Note that scripts cannot find out if 2 events are grouped
 * or not (a serious lack in the scripting language), so split/remove
 * actions are applied to ALL events that "seem" to belong together. 2
 * events are considered as "seeming to belong together" if they belong
 * to different tracks, but start and end at about the same timecode.
 *
 * Algorithm: Starting at first track and first event, events are
 * examined until an event is either "under" the cursor, or at its "left".
 * "under" means: starting at less than 3 frames left  of cursor
 *            and ending   at more than 3 frames right of cursor,
 * "at its left" means: ending within +/- 3 frames around cursor.
 *
 * In the first case, the event plus all "seemingly-belonging-to-it" events
 * (see above) are split at the cursor position.
 *
 * In the latter case, the "left-event" plus all "seemingly-belonging-to-it"
 * events (see above) are removed + rippled (only tracks with such events
 * will be rippled, other tracks remain unaffected).
 */

import System.Windows.Forms;
import Sony.Vegas;

/*
 * In order to find out if cursor is "between" 2 events (at edge),
 * we allow a tolerance of one frame in each direction:
 */


var tolerance = Vegas.Cursor;
tolerance.FromString("00:00:00,03");

var cursorFrom = Vegas.Cursor;
var cursorTo   = Vegas.Cursor;
cursorFrom -= tolerance;
cursorTo += tolerance;


/* the first event where cursor is above or at its right edge "freezes"
   script into split mode (cursor above event) or remove mode (cursor at
   events right edge). Initially, this mode is still open, so both flags
   are set to false:
*/
var removeMode = false;
var splitMode  = false;

/* After mode is set, the length of first split/removed event is memorized.
   Later split/event actions are only applied to events of
   same length (+/- tolerance):
*/


var evntEnd   = tolerance;   /* any timecode for init */
var evntStart = tolerance;   /* any timecode for init */
var track,evnt,evnt2,thisStart,thisEnd,sameStart,sameEnd,isEquivalent;


try {
  for (track in Vegas.Project.Tracks) {
    eventloop:
    for (evnt in track.Events) {
      /* set this events start and end (they are reused several times) */
      thisStart = evnt.Start;
      thisEnd   = evnt.Start + evnt.Length;

      /* find out if event seems to belong to teh first split/removed event
         (optimally, using grouping mechanism - not available in current
         scripting, so we use a heuristic)
      */
      sameStart = false; sameEnd = false; isEquivalent = false;
      if ((thisStart + tolerance > evntStart)  &&
          (thisStart - tolerance < evntStart)) sameStart=true;
      if ((thisEnd   + tolerance > evntEnd)    &&
          (thisEnd   - tolerance < evntEnd))   sameEnd=true;
      if (sameStart && sameEnd) isEquivalent=true;

      /* first case: split event, if
         - we are not in remove-mode,
	 - and we're clearly inside the event (minus tolerance),
	 - and, if events were split before, event is equivalent to first split one:
      */
      if (!removeMode &&
          thisStart < cursorFrom &&
          thisEnd   > cursorTo   &&
          (!splitMode || isEquivalent ))
      {
        /* set split-mode, remember length, and split event: */
        if (!splitMode) {
	  evntStart = thisStart;
          evntEnd   = thisEnd;
	  splitMode = true;
	}
	evnt2 = evnt.Split(Vegas.Cursor - thisStart);
        break eventloop; /* only one event per track is split/removed */
      }
      /* ignore event if we're in split-mode; rest of loop body is
         about removing events */
      if (splitMode) continue;

      /* for the rest of the loop body, we're either in remove-mode, or
         still open (neither split-mode nor remove-mode). So we must check
	 if the event is a candidate for removal, meaning cursor is at its
	 right edge (+/- tolerance): */
      if (thisEnd < cursorFrom) continue;
      if (thisEnd > cursorTo) continue;

      /* Cursor is at right edge of event... if we already removed events
         earlier (removeEvents is true), make sure that length matches
      */
      if ( removeMode  && !isEquivalent) continue;

      /* first, ripple left all same-track-events that are clearly to the right */
      for (evnt2 in track.Events)
        if (evnt2.Start >= cursorFrom)
          evnt2.AdjustStartLength(evnt2.Start-evnt.Length, evnt2.Length, true);
      /* if we're still in open mode: set remove-mode, remember length
         and adjust cursor
      */
      if (!removeMode) {
        evntStart = thisStart;
        evntEnd   = thisEnd;
	removeMode = true;
        /* cursor will finally be at left of all removed events: */
        Vegas.Cursor = thisStart;
      }

      track.Events.Remove(evnt);
      break eventloop;  /* only one event per track is split/removed */
    }
  }
} catch (e) {MessageBox.Show(e);}
